/**********************************************************************
 * Copyright (c) 2008, 2013 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: SSLFactory.java,v 1.2 2008-01-25 19:37:28 jcayne Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.tptp.platform.agentcontroller.jsse;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.UnknownHostException;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.Provider;
import java.security.Security;
import java.security.Provider.Service;

import javax.net.ssl.KeyManager;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;

public class SSLFactory {
	private static SSLFactory _instance = null;
	
	private SSLContext _sslContext = null;
	private boolean _sslContextInitialized = false; 
	
	private KeyManager[] _kms = new KeyManager[1];
	private TrustManager[] _tms = new TrustManager[1];
	
	// Security Protocol related variables
	private String _protocol="SSL";
	private String _userProvidedProtocol;

	public synchronized static SSLFactory getInstance() throws NoSuchAlgorithmException {
		if(_instance == null) {
			_instance = new SSLFactory();
		}

		return _instance;
	}

	private SSLFactory() throws NoSuchAlgorithmException {
		/* Determine the acceptable protocol */
		String userProvidedProtocol = System.getenv("SECURITY_SOCKET_PROTOCOL");
		String protocol = getProtocol(userProvidedProtocol);
		
		_sslContext = SSLContext.getInstance(protocol);
	}

	public SSLSocket createSocket(Socket socket) {
		SSLSocketFactory factory;
		SSLSocket sslSocket = null;

		// Initialize the SSL context only once
		synchronized(_sslContext) {
			if(!_sslContextInitialized) {
				try {
					_sslContext.init(_kms, _tms, null);
					_sslContextInitialized = true;
				} catch(Exception e) {
					e.printStackTrace();
				}
			}
		}

		factory = _sslContext.getSocketFactory();
		
		// An alternative here was: factory = (SSLSocketFactory) SSLSocketFactory.getDefault();
		// This would have involved using anonymous ciphers.


		try {
			sslSocket = (SSLSocket) factory.createSocket(socket, socket.getInetAddress().getHostName(), socket.getPort(), true);
			
		} catch (IOException e) {
				//	e.printStackTrace();
		}

		return sslSocket;
	}

	public Socket getClientSocket(String host, int port) throws IOException, KeyManagementException, UnknownHostException {
		Socket socket = null;
		
		// Initialize the SSL context only once
		synchronized(_sslContext) {
			if(!_sslContextInitialized) {
				_sslContext.init(_kms, _tms, null);
				_sslContextInitialized = true;
			}
		}
		
		SSLSocketFactory factory = _sslContext.getSocketFactory();

		if(factory != null) {
			socket = factory.createSocket(host, port);
		}

		return socket;
	}

	public ServerSocket getServerSocket(int port) throws IOException, KeyManagementException {
		ServerSocket socket = null;
		
		// Initialize the SSL context only once
		synchronized(_sslContext) {
			if(!_sslContextInitialized) {
				_sslContext.init(_kms, _tms, null);
				_sslContextInitialized = true;
			}
		}
		
		SSLServerSocketFactory factory = _sslContext.getServerSocketFactory();

		if(factory != null) {
			socket = factory.createServerSocket(port);
		}

		return socket;
	}

	public void setKeyManager(KeyManager km) {
		_kms[0] = km;
	}

	public void setTrustManager(TrustManager tm) {
		_tms[0] = tm;
	}

	// Security protocol related code
	/*  
	 * Note: The exact code exists in the following three classes:
	 * 			org.eclipse.tptp.platform.agentcontroller.jsse.SSLFactory
	 * 			org.eclipse.tptp.platform.execution.client.core.internal.SecureConnectionImpl
	 * 			org.eclipse.hyades.internal.execution.local.control.SecurityProtocol
	 * Any changes made to the following methods need to be replicated in all three classes.
	 * 
	 */
	
	public String getProtocol(String userProtocol) {
		_userProvidedProtocol = userProtocol;
		
		if (_userProvidedProtocol!=null) {
			// use user supplied protocol if it is valid
			_protocol = validate(_userProvidedProtocol) ? _userProvidedProtocol : _protocol;
		}
		else {
			_protocol = (isIBMJVM() && validate("SSL_TLSv2")) ? "SSL_TLSv2" : _protocol;
		}
		
		return _protocol;
	}
	
	/**
	 * Checks if the protocol is supported by the JVM used by the agent controller 
	 */
	public boolean validate(String protocol) {
		//ArrayList<String> protocolsList = getJVMSupportedProtocols();
		
		// Check for any of the other valid algorithms
		if(protocol.equalsIgnoreCase("default")) {
			return false;
		}		
		
		Provider[] providers = Security.getProviders();
		for (int p = 0; p < providers.length; p++) {
			Provider provider = providers[p];
			//System.out.println("Provider[" + p + "] = " + providers[p]);
			// see if this provider supports the provided security protocol
			Service service = provider.getService(SSLContext.class.getSimpleName(), protocol);
			if(service != null) {
				return true;
			}
		}
		return false;
	}
	
	public boolean isIBMJVM(){
		if (System.getProperty("java.vm.vendor").indexOf("IBM") >= 0) {
			//String version = System.getProperty("java.version");
			return true;
		}
		return false;
	}

}
