/*******************************************************************************
 * Copyright (c) 2011 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: 
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.hyades.trace.ui.internal.export;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Date;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.hyades.models.hierarchy.TRCAgent;
import org.eclipse.hyades.models.hierarchy.TRCAgentProxy;
import org.eclipse.hyades.trace.ui.internal.wizard.TraceWizardMessages;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.tptp.platform.common.ui.trace.internal.CommonUITraceImages;
import org.eclipse.tptp.platform.common.ui.trace.internal.CommonUITracePlugin;
import org.eclipse.ui.IExportWizard;
import org.eclipse.ui.IWorkbench;

import com.ibm.icu.text.DateFormat;

public class TraceExportWizard extends Wizard implements IExportWizard {

	IWorkbench _workbench;
	IStructuredSelection _selection;
	
	TraceExportWizardPage1 _mainPage;
	
	public void init(IWorkbench w, IStructuredSelection ss) {
		_workbench = w;
		_selection = ss;
		
		setWindowTitle(TraceWizardMessages.EXPORT_WIZARD_WINDOW_TITLE);
		
		_mainPage = new TraceExportWizardPage1(_selection);
		
		setDefaultPageImageDescriptor(CommonUITraceImages.INSTANCE.getImageDescriptor(CommonUITraceImages.IMG_UI_WZ_EXPORT_FILE));
	}
	
	public void addPages() {
		addPage(_mainPage);
	}

	private String getDumpFileName(Date d, boolean isBinary){
		String timestamp = DateFormat.getDateTimeInstance(DateFormat.FULL, DateFormat.FULL).format(d).replace(' ', '_').replace(':', '_').replace(',', '_');
		return "TraceContents-" + timestamp + (isBinary ? ".bin" : ".xml"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}
	
	public static boolean isBinaryOutputFormat() {
		final String TI_VARIABLE_OUTPUT_FORMAT="TPTP_OUTPUT_FORMAT"; // Can be XML or BINARY  //$NON-NLS-1$
		String format = System.getenv(TI_VARIABLE_OUTPUT_FORMAT);
		boolean isBinary = format != null ? (!format.equalsIgnoreCase("xml")) : true; //$NON-NLS-1$
		return isBinary;
	}

	public boolean performFinish() {
		final TRCAgent aa = _mainPage.getSelectedAgent();
		final String file = _mainPage.getFilename();
		
		final Job job = new Job(TraceWizardMessages.EXPORT_WIZARD_EXP_TO_FILE) {

			@Override
			protected IStatus run(IProgressMonitor m) {
				
				TRCAgentProxy ap = aa.getAgentProxy();
				File fileF;
				
				Date date = new Date((long) aa.getStartTime() * 1000);
				
				try {
					boolean isBinary = isBinaryOutputFormat();
					
					fileF = new File(file);
					OutputStream os = new FileOutputStream(fileF);
					
					ZipOutputStream zout = new ZipOutputStream(os);
					ZipEntry ze = new ZipEntry(getDumpFileName(date, isBinary));
					zout.putNextEntry(ze);
					
					
					long start = System.currentTimeMillis();
					TraceOutputUtil.exportAgentDataToStream(aa, zout, m, isBinary);
					
					zout.close();
					os.close();
					
//					System.out.println("Elapsed Time:"+(float)(System.currentTimeMillis()-start)/1000f); //$NON-NLS-1$
					
					if(m.isCanceled()) {
						fileF.delete();
						return Status.CANCEL_STATUS;
					}

					return Status.OK_STATUS;
					
				} catch(IOException e) {
					final IOException f = e;
					_workbench.getDisplay().asyncExec(new Runnable() {

						public void run() {
							Status err = new Status(Status.WARNING, CommonUITracePlugin.PLUGIN_ID, f.getMessage());
							ErrorDialog.openError(_workbench.getDisplay().getActiveShell(), TraceWizardMessages.EXPORT_WIZARD_ERR_IO_TITLE, TraceWizardMessages.EXPORT_WIZARD_ERR_IO_MSG, err);							
						} 
						
					});
										
					return Status.OK_STATUS;
				}

			}
			
		};
		job.setUser(true);
		
		job.schedule();
				
		return true;
	}

	@Override
	public boolean canFinish() {
		return _mainPage.isPageComplete();
	}
	
	
}
